"""cli
Command Line Interface for euler-workbench.

Copyright (C) 2014  Stijn Wouters  <w.stijn@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Last modified: 01 August 2014.
By: Stijn Wouters.
"""
import os

import click
from tabulate import tabulate

from eulerworkbench import eulerworkbench as ewb


pass_workbench = click.make_pass_decorator(ewb.EulerWorkbench, ensure=True)


@click.group()
@click.version_option(prog_name='euler-workbench')
@pass_workbench
def eulerworkbench(workbench):
    """Command Line application for your Project Euler.
    """
    return


@eulerworkbench.command()
@click.argument('path', nargs=-1, type=click.Path(exists=True))
@click.option('--output', '-o', default='-', type=click.File('w'))
@click.option('--format', '-f', default='simple', show_default=True,
              type=click.Choice(['plain', 'simple', 'grid', 'pipe', 'orgtbl',
                                 'rst', 'mediawiki', 'latex']),
              help='Table format.')
@click.option('--precision', default=6, show_default=True,
              type=click.IntRange(0, None),
              help='Numbers of digits after decimal point.')
@pass_workbench
def benchmark(workbench, path, output, format, precision):
    """Benchmark your Project Euler solutions.

    This will show a table with the name of the solution function, accompanied
    with the answer and the benchmark. A smaller benchmark indicates a faster
    implementation, but it could also be NAN, indicating that Python ran out of
    memory. In that case, the answer will be left empty, since the answer is
    too suspicious.
    """
    # use current directory as default path
    if 0 == len(path):
        path = os.getcwd()

    benchmarks = []

    # discover solutions
    ewb.EulerWorkbench.discover(path)

    # show progresbar while benchmarking
    with click.progressbar(workbench.benchmarks(),
                           length=len(workbench.solutions),
                           fill_char='.', empty_char=' ') as bmarks:
        for name, answer, bench in bmarks:
            benchmarks.append([name, answer, bench])

    # generate table
    table = tabulate(benchmarks, headers=('SOLUTION', 'ANSWER', 'BENCHMARK'),
                     tablefmt=format, numalign='right',
                     floatfmt='.{}F'.format(precision))

    click.echo(table, file=output)
    return
