"""eulerworkbench
All workbench are build on top of this module.

Copyright (C) 2014  Stijn Wouters  <w.stijn@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

Last modified: 01 August 2014.
By: Stijn Wouters.
"""
import os
import sys
import pkgutil
import importlib.machinery as importer
from time import perf_counter


def sourcepath(path):
    """Source python modules from the given path. The path can be either a path
    to a module, or a path to a directory. If a directory is given, it will
    search into that directory for any modules or packages.

    Note that this does not search recursively into a directory, unless it's a
    package.
    """
    if not os.path.exists(path):
        msg = 'File or directory not found: {}'.format(path)
        raise FileNotFoundError(msg)
    elif os.path.isfile(path):
        # get module name
        module, _ = os.path.splitext(os.path.basename(path))

        # stop if module already sourced
        if module in sys.modules:
            return

        # load source
        loader = importer.SourceFileLoader(module, path)
        loader.load_module()
    elif os.path.isdir(path):
        # try to iterate over the modules on the given path
        iter_modules = pkgutil.iter_modules(path=[path])

        for finder, module, is_package in iter_modules:
            if is_package:
                package_path = os.path.join(finder.path, module)

                sourcepath(package_path)
            else:
                # skip if module already sourced
                if module in sys.modules:
                    continue

                # get path to module
                module_path = os.path.join(finder.path, module + '.py')

                # load source
                loader = importer.SourceFileLoader(module, module_path)
                loader.load_module()
    else:
        # The given pathname do exist, but is not a directory or a regular file
        # (this could be a symbolic link or a broken file for example).
        msg = 'File is neither a directory or a file {}'.format(path)
        raise OSError(msg)


class EulerWorkbench:
    """Base workbench providing decorators and basic tools for your Project
    Euler.
    """

    SOLUTIONS = []
    """Shared database containing all your solutions on Project Euler (as found
    so far).
    """

    def __init__(self):
        """Constructor. Initialize default settings.
        """
        # alias to static data member, just for easy typing
        self.solutions = EulerWorkbench.SOLUTIONS
        return

    @staticmethod
    def solution(solution):
        """Decorator marking the function as a solution for a problem that
        was posed on Project Euler.

        Requires the solution to be callable (otherwise `AttributeError` is
        raised).
        """
        if callable(solution):
            EulerWorkbench.SOLUTIONS.append(solution)
        else:
            raise AttributeError('{} is not callable'.format(solution))

    @staticmethod
    def discover(path=os.getcwd()):
        """Discover Project Euler solutions.
        """
        # Convert string to iterables, so you can still use this
        # implementation, instead of jumpint to somewhere else.
        if isinstance(path, str):
            path = [path]

        for pathname in path:
            sourcepath(pathname)

    def benchmarks(self):
        """Benchmarks the solutions.

        Yields a tuple of (<qualified function name>, answer, benchmark), where
        the benchmark is a return value of `time.perf_counter`:

            Returns the value (in fractional seconds) of a performance counter,
            i.e. a clock with the highest available resolution to measure a
            short duration. It does include time elapsed during sleep and is
            system-wide. The reference point of the returned value is
            undefined, so that only the difference between the result of
            consecutive calls is valid.

        Note that, when the memory is exhausted (raising `MemoryError`, the
        answer will be set to `None` and the benchmark to `NaN`.
        """
        for solution in self.solutions:

            try:
                start = perf_counter()
                answer = solution()
                end = perf_counter()

                yield (solution.__qualname__, answer, end - start)
            except MemoryError:
                # memory exhausted
                yield (solution.__qualname__, None, float('nan'))
